@@ -1,4 +1,4 @@
-"""Authors: Saksham Sharda and Alessio Buccino."""
+"""Authors: Heberto Mayorquin, Saksham Sharda and Alessio Buccino, Szonja Weigl"""
 import os
 from pathlib import Path
 from warnings import warn
@@ -15,6 +15,7 @@ from pynwb.image import GrayscaleImage
 from pynwb.device import Device
 from pynwb.ophys import (
     ImageSegmentation,
+    ImagingPlane,
     Fluorescence,
     OpticalChannel,
     TwoPhotonSeries,
@@ -39,45 +40,46 @@ def get_default_ophys_metadata():
 
     default_device = dict(name="Microscope")
 
+    default_optical_channel = dict(
+        name="OpticalChannel",
+        emission_lambda=np.nan,
+        description="no description",
+    )
+
+    default_imaging_plane = dict(
+        name="ImagingPlane",
+        description="no description",
+        excitation_lambda=np.nan,
+        indicator="unknown",
+        location="unknown",
+        device=default_device["name"],
+        optical_channel=[default_optical_channel],
+    )
+
+    default_roi_response_series = dict(
+        name="RoiResponseSeries",
+        description="array of raw fluorescence traces",
+    )
+
+    default_fluoresence = dict(roi_response_series=[default_roi_response_series])
+
+    default_two_photon_series = dict(
+        name="TwoPhotonSeries",
+        description="no description",
+        comments="Generalized from RoiInterface",
+        unit="n.a.",
+    )
+
     metadata.update(
         Ophys=dict(
             Device=[default_device],
-            Fluorescence=dict(
-                roi_response_series=[
-                    dict(
-                        name="RoiResponseSeries",
-                        description="array of raw fluorescence traces",
-                    )
-                ]
-            ),
+            Fluorescence=default_fluoresence,
             ImageSegmentation=dict(plane_segmentations=[dict(description="Segmented ROIs", name="PlaneSegmentation")]),
-            ImagingPlane=[
-                dict(
-                    name="ImagingPlane",
-                    description="no description",
-                    excitation_lambda=np.nan,
-                    indicator="unknown",
-                    location="unknown",
-                    device=default_device["name"],
-                    optical_channel=[
-                        dict(
-                            name="OpticalChannel",
-                            emission_lambda=np.nan,
-                            description="no description",
-                        )
-                    ],
-                )
-            ],
-            TwoPhotonSeries=[
-                dict(
-                    name="TwoPhotonSeries",
-                    description="no description",
-                    comments="Generalized from RoiInterface",
-                    unit="n.a.",
-                )
-            ],
+            ImagingPlane=[default_imaging_plane],
+            TwoPhotonSeries=[default_two_photon_series],
         ),
     )
+
     return metadata
 
 
@@ -124,41 +126,98 @@ def get_nwb_imaging_metadata(imgextractor: ImagingExtractor):
 
 
 def add_devices(nwbfile: NWBFile, metadata: dict) -> NWBFile:
-    """Add optical physiology devices from metadata."""
+    """
+    Add optical physiology devices from metadata.
+    The metadata concerning the optical physiology should be stored in metadata["Ophys]["Device"]
+    This function handles both a text specification of the device to be built and an actual pynwb.Device object.
+
+    """
     metadata_copy = deepcopy(metadata)
     default_metadata = get_default_ophys_metadata()
     metadata_copy = dict_deep_update(default_metadata, metadata_copy, append_list=False)
     device_metadata = metadata_copy["Ophys"]["Device"]
 
     for device in device_metadata:
-        device = Device(**device) if isinstance(device, dict) else device
-        if device.name not in nwbfile.devices:
+        device_name = device["name"] if isinstance(device, dict) else device.name
+        if device_name not in nwbfile.devices:
+            device = Device(**device) if isinstance(device, dict) else device
             nwbfile.add_device(device)
 
     return nwbfile
 
 
-def _add_imaging_plane(nwbfile: NWBFile, metadata=dict) -> NWBFile:
+def _create_imaging_plane_from_metadata(nwbfile: NWBFile, imaging_plane_metadata: dict) -> ImagingPlane:
+    """
+    Private auxiliar function to create an ImagingPlane object from pynwb using the imaging_plane_metadata
+    Parameters
+    ----------
+    nwbfile : NWBFile
+        An previously defined -in memory- NWBFile.
 
-    metadata_copy = deepcopy(metadata)
-    add_devices(nwbfile=nwbfile, metadata=metadata_copy)
+    imaging_plane_metadata : dict
+        The metadata to create the ImagingPlane object.
 
-    # Add the image plane
-    image_plane_metadata = metadata_copy["Ophys"]["ImagingPlane"][0]
+    Returns
+    -------
+    ImagingPlane
+        The created ImagingPlane.
+    """
 
-    device_name = image_plane_metadata["device"]
-    image_plane_metadata["device"] = nwbfile.devices[device_name]
+    device_name = imaging_plane_metadata["device"]
+    imaging_plane_metadata["device"] = nwbfile.devices[device_name]
 
-    image_plane_metadata["optical_channel"] = [
-        OpticalChannel(**metadata) for metadata in image_plane_metadata["optical_channel"]
+    imaging_plane_metadata["optical_channel"] = [
+        OpticalChannel(**metadata) for metadata in imaging_plane_metadata["optical_channel"]
     ]
 
-    nwbfile.create_imaging_plane(**image_plane_metadata)
+    imaging_plane = ImagingPlane(**imaging_plane_metadata)
+
+    return imaging_plane
+
+
+def add_imaging_plane(nwbfile: NWBFile, metadata: dict, imaging_plane_index: int = 0) -> NWBFile:
+    """
+    Adds the imaging plane specificied by the metadata to the nwb file.
+    The imaging plane that is added is the one located in metadata["Ophys"]["ImagingPlane"][imaging_plane_index]
+
+    Parameters
+    ----------
+    nwbfile : NWBFile
+        An previously defined -in memory- NWBFile.
+    metadata : dict
+        The metadata in the nwb conversion tools format.
+    imaging_plane_index : int, optional
+        The metadata in the nwb conversion tools format is a list of the different imaging planes to add.
+        Specificy which element of the list with this parameter, by default 0
+
+    Returns
+    -------
+    NWBFile
+        The nwbfile passed as an input with the imaging plane added.
+    """
+
+    # Set the defaults and required infrastructure
+    metadata_copy = deepcopy(metadata)
+    default_metadata = get_default_ophys_metadata()
+    metadata_copy = dict_deep_update(default_metadata, metadata_copy, append_list=False)
+    add_devices(nwbfile=nwbfile, metadata=metadata_copy)
+
+    imaging_plane_metadata = metadata_copy["Ophys"]["ImagingPlane"][imaging_plane_index]
+    imaging_plane_name = imaging_plane_metadata["name"]
+    existing_imaging_planes = nwbfile.imaging_planes
+
+    if imaging_plane_name not in existing_imaging_planes:
+        imaging_plane = _create_imaging_plane_from_metadata(
+            nwbfile=nwbfile, imaging_plane_metadata=imaging_plane_metadata
+        )
+        nwbfile.add_imaging_plane(imaging_plane)
 
     return nwbfile
 
 
-def add_two_photon_series(imaging, nwbfile, metadata, buffer_size=10, use_times=False):
+def add_two_photon_series(
+    imaging, nwbfile, metadata, buffer_size=10, use_times=False, two_photon_series_index: int = 0
+):
     """
     Auxiliary static method for nwbextractor.
 
@@ -172,15 +231,18 @@ def add_two_photon_series(imaging, nwbfile, metadata, buffer_size=10, use_times=
     metadata_copy = dict_deep_update(get_nwb_imaging_metadata(imaging), metadata_copy, append_list=False)
 
     # Tests if TwoPhotonSeries already exists in acquisition
-    two_photon_series_metadata = metadata_copy["Ophys"]["TwoPhotonSeries"][0]
+    two_photon_series_metadata = metadata_copy["Ophys"]["TwoPhotonSeries"][two_photon_series_index]
     two_photon_series_name = two_photon_series_metadata["name"]
 
     if two_photon_series_name in nwbfile.acquisition:
         warn(f"{two_photon_series_name} already on nwbfile")
         return nwbfile
 
-    # Add the image plane to nwb.
-    nwbfile = _add_imaging_plane(nwbfile=nwbfile, metadata=metadata_copy)
+    # Add the image plane to nwb
+    nwbfile = add_imaging_plane(nwbfile=nwbfile, metadata=metadata_copy)
+    imaging_plane_name = two_photon_series_metadata["imaging_plane"]
+    imaging_plane = nwbfile.get_imaging_plane(name=imaging_plane_name)
+    two_photon_series_metadata.update(imaging_plane=imaging_plane)
 
     # Add the data
     def data_generator(imaging):
@@ -206,11 +268,6 @@ def add_two_photon_series(imaging, nwbfile, metadata, buffer_size=10, use_times=
         two_p_series_kwargs.update(timestamps=H5DataIO(timestamps, compression="gzip"))
         two_p_series_kwargs["rate"] = None
 
-    # Add imaging plane
-    imaging_plane_name = two_photon_series_metadata["imaging_plane"]
-    imaging_plane = nwbfile.get_imaging_plane(name=imaging_plane_name)
-    two_p_series_kwargs.update(imaging_plane=imaging_plane)
-
     # Add the TwoPhotonSeries to the nwbfile
     two_photon_series = TwoPhotonSeries(**two_p_series_kwargs)
     nwbfile.add_acquisition(two_photon_series)
@@ -401,6 +458,7 @@ def write_segmentation(
         if metadata is not None and not isinstance(metadata, list):
             metadata = [metadata]
     metadata_base_list = [get_nwb_segmentation_metadata(sgobj) for sgobj in segext_objs]
+
     # updating base metadata with new:
     for num, data in enumerate(metadata_base_list):
         metadata_input = metadata[num] if metadata else {}
@@ -430,18 +488,18 @@ def write_segmentation(
             nwbfile = io.read()
         else:
             nwbfile = make_nwbfile_from_metadata(metadata=metadata_base_common)
-    # Subject:
-    if metadata_base_common.get("Subject") and nwbfile.subject is None:
-        nwbfile.subject = Subject(**metadata_base_common["Subject"])
+
     # Processing Module:
     if "ophys" not in nwbfile.processing:
         ophys = nwbfile.create_processing_module("ophys", "contains optical physiology processed data")
     else:
         ophys = nwbfile.get_processing_module("ophys")
+
     for plane_no_loop, (segext_obj, metadata) in enumerate(zip(segext_objs, metadata_base_list)):
-        # Device:
-        if metadata["Ophys"]["Device"][0]["name"] not in nwbfile.devices:
-            nwbfile.create_device(**metadata["Ophys"]["Device"][0])
+
+        # Add device:
+        add_devices(nwbfile=nwbfile, metadata=metadata)
+
         # ImageSegmentation:
         image_segmentation_name = (
             "ImageSegmentation" if plane_no_loop == 0 else f"ImageSegmentation_Plane{plane_no_loop}"
@@ -451,23 +509,12 @@ def write_segmentation(
             ophys.add(image_segmentation)
         else:
             image_segmentation = ophys.data_interfaces.get(image_segmentation_name)
-        # OpticalChannel:
-        optical_channels = [OpticalChannel(**i) for i in metadata["Ophys"]["ImagingPlane"][0]["optical_channel"]]
 
-        # ImagingPlane:
-        image_plane_name = "ImagingPlane" if plane_no_loop == 0 else f"ImagePlane_{plane_no_loop}"
-        if image_plane_name not in nwbfile.imaging_planes.keys():
-            input_kwargs = dict(
-                name=image_plane_name,
-            )
-            metadata["Ophys"]["ImagingPlane"][0]["optical_channel"] = optical_channels
-            input_kwargs.update(**metadata["Ophys"]["ImagingPlane"][0])
-            input_kwargs.update(device=nwbfile.get_device(metadata_base_common["Ophys"]["Device"][0]["name"]))
-            if "imaging_rate" in input_kwargs:
-                input_kwargs["imaging_rate"] = float(input_kwargs["imaging_rate"])
-            imaging_plane = nwbfile.create_imaging_plane(**input_kwargs)
-        else:
-            imaging_plane = nwbfile.imaging_planes[image_plane_name]
+        # Add imaging plane
+        imaging_plane_name = "ImagingPlane" if plane_no_loop == 0 else f"ImagePlane_{plane_no_loop}"
+        add_imaging_plane(nwbfile=nwbfile, metadata=metadata)
+        imaging_plane = nwbfile.imaging_planes[imaging_plane_name]
+
         # PlaneSegmentation:
         input_kwargs = dict(
             description="output from segmenting imaging plane",
@@ -567,6 +614,7 @@ def write_segmentation(
         # create Two Photon Series:
         if "TwoPhotonSeries" not in nwbfile.acquisition:
             warn("could not find TwoPhotonSeries, using ImagingExtractor to create an nwbfile")
+
         # adding images:
         images_dict = segext_obj.get_images_dict()
         if any([image is not None for image in images_dict.values()]):
