@@ -1,5 +1,4 @@
-"""Author: Cody Baker."""
-from logging import warning
+"""Author: Heberto Mayorquin, Cody Baker."""
 import uuid
 import warnings
 import numpy as np
@@ -16,9 +15,10 @@ from spikeextractors import RecordingExtractor, SortingExtractor
 from numbers import Real
 from hdmf.data_utils import DataChunkIterator
 from hdmf.backends.hdf5.h5_utils import H5DataIO
+import psutil
 
 from .spikeinterfacerecordingdatachunkiterator import SpikeInterfaceRecordingDataChunkIterator
-from ..nwb_helpers import get_module, make_nwbfile_from_metadata, make_or_load_nwbfile
+from ..nwb_helpers import get_module, make_or_load_nwbfile
 from ...utils import dict_deep_update, OptionalFilePathType
 
 SpikeInterfaceRecording = Union[BaseRecording, RecordingExtractor]
@@ -458,6 +458,35 @@ def add_electrodes(
         warnings.warn(f"No information added to the electrodes table")
 
 
+def check_if_recording_traces_fit_into_memory(recording: SpikeInterfaceRecording, segment_index: int = 0) -> None:
+    """Raises an error if the full traces of a recording extractor are larger than psutil.virtual_memory().available
+
+    Parameters
+    ----------
+    recording : SpikeInterfaceRecording
+        A recording extractor object from spikeinterface.
+    segment_index : int, optional
+        The segment index of the recording extractor object, by default 0
+
+    Raises
+    ------
+    MemoryError
+    """
+    element_size_in_bytes = recording.get_dtype().itemsize
+    num_channels = recording.get_num_channels()
+    num_frames = recording.get_num_frames(segment_index=segment_index)
+
+    traces_size_in_bytes = element_size_in_bytes * num_channels * num_frames
+    available_memory_in_bytes = psutil.virtual_memory().available
+
+    if traces_size_in_bytes > available_memory_in_bytes:
+        message = (
+            f"Memory error, full electrical series is {round(traces_size_in_bytes/1e9, 2)} GB) but only"
+            f"({round(available_memory_in_bytes/1e9, 2)} GB are available. Use iterator_type='V2'"
+        )
+        raise MemoryError(message)
+
+
 def add_electrical_series(
     recording: SpikeInterfaceRecording,
     nwbfile=None,
@@ -470,7 +499,7 @@ def add_electrical_series(
     write_scaled: bool = False,
     compression: Optional[str] = "gzip",
     compression_opts: Optional[int] = None,
-    iterator_type: Optional[str] = None,
+    iterator_type: Optional[str] = "v2",
     iterator_opts: Optional[dict] = None,
 ):
     """
@@ -540,6 +569,7 @@ def add_electrical_series(
 
     if not nwbfile.electrodes:
         add_electrodes(recording, nwbfile, metadata)
+
     assert write_as in [
         "raw",
         "processed",
@@ -562,13 +592,11 @@ def add_electrical_series(
         eseries_kwargs = dict(
             name="ElectricalSeries_raw",
             description="Raw acquired data",
-            comments="Generated from SpikeInterface::NwbRecordingExtractor",
         )
     elif write_as == "processed":
         eseries_kwargs = dict(
             name="ElectricalSeries_processed",
             description="Processed data",
-            comments="Generated from SpikeInterface::NwbRecordingExtractor",
         )
         ecephys_mod = get_module(
             nwbfile=nwbfile,
@@ -581,7 +609,6 @@ def add_electrical_series(
         eseries_kwargs = dict(
             name="ElectricalSeries_lfp",
             description="Processed data - LFP",
-            comments="Generated from SpikeInterface::NwbRecordingExtractor",
         )
         ecephys_mod = get_module(
             nwbfile=nwbfile,
@@ -590,9 +617,11 @@ def add_electrical_series(
         )
         if "LFP" not in ecephys_mod.data_interfaces:
             ecephys_mod.add(pynwb.ecephys.LFP(name="LFP"))
+
     if metadata is not None and "Ecephys" in metadata and es_key is not None:
         assert es_key in metadata["Ecephys"], f"metadata['Ecephys'] dictionary does not contain key '{es_key}'"
         eseries_kwargs.update(metadata["Ecephys"][es_key])
+
     if write_as == "raw":
         assert (
             eseries_kwargs["name"] not in nwbfile.acquisition
@@ -613,6 +642,8 @@ def add_electrical_series(
     else:
         channel_indices = checked_recording.ids_to_indices(channel_name_array)
 
+    add_electrodes(recording=recording, nwbfile=nwbfile, metadata=metadata)
+
     table_ids = [list(nwbfile.electrodes.id[:]).index(id) for id in channel_indices]
 
     electrode_table_region = nwbfile.create_electrode_table_region(
@@ -633,7 +664,11 @@ def add_electrical_series(
         else:
             eseries_kwargs.update(conversion=1e-6)
             eseries_kwargs.update(channel_conversion=channel_conversion)
-    if iterator_type is None or iterator_type == "v2":
+    if iterator_type is None:
+        check_if_recording_traces_fit_into_memory(recording=checked_recording, segment_index=segment_index)
+        ephys_data = checked_recording.get_traces(return_scaled=write_scaled, segment_index=segment_index)
+
+    elif iterator_type == "v2":
         ephys_data = SpikeInterfaceRecordingDataChunkIterator(
             recording=checked_recording,
             segment_index=segment_index,
@@ -653,6 +688,7 @@ def add_electrical_series(
         raise NotImplementedError(f"iterator_type ({iterator_type}) should be either 'v1' or 'v2' (recommended)!")
     eseries_kwargs.update(data=H5DataIO(data=ephys_data, compression=compression, compression_opts=compression_opts))
 
+    # Tiemestamps vs rate (#To-do: should use check_regular_timestamps)
     if not use_times and starting_time is None:
         eseries_kwargs.update(starting_time=float(checked_recording.get_times(segment_index=segment_index)[0]))
     elif not use_times and starting_time is not None:
@@ -682,6 +718,8 @@ def add_electrical_series(
                 compression_opts=compression_opts,
             )
         )
+
+    # Create ElectricalSeries object and add it to nwbfile
     es = pynwb.ecephys.ElectricalSeries(**eseries_kwargs)
     if write_as == "raw":
         nwbfile.add_acquisition(es)
@@ -871,7 +909,7 @@ def write_recording(
     write_scaled: bool = False,
     compression: Optional[str] = None,
     compression_opts: Optional[int] = None,
-    iterator_type: Optional[str] = None,
+    iterator_type: Optional[str] = "v2",
     iterator_opts: Optional[dict] = None,
     save_path: OptionalFilePathType = None,  # TODO: to be removed
 ):
